BluePallas

Struct BluePallas 

pub struct BluePallas<M>(pub(crate) PhantomData<M>);
The BluePallas ciphersuite, which uses the Pallas curve and Poseidon hash function.

Note that this ciphersuite is used for FROST signatures in the Mina protocol and has a lot of Mina-specific logic This library SHOULD not be treated as a general-purpose BluePallas ciphersuite, but rather as a Mina-specific implementation.

Tuple Fields

0: PhantomData<M>