pub type SigningPackage<M> = SigningPackage<BluePallas<M>>;Expand description
Generated by the coordinator of the signing operation and distributed to each signing party.
Aliased Type
pub struct SigningPackage<M> { /* private fields */ }pub type SigningPackage<M> = SigningPackage<BluePallas<M>>;Generated by the coordinator of the signing operation and distributed to each signing party.
pub struct SigningPackage<M> { /* private fields */ }