pub struct BluePallas<M>(pub(crate) PhantomData<M>);Expand description
The BluePallas ciphersuite, which uses the Pallas curve and Poseidon hash function.
Note that this ciphersuite is used for FROST signatures in the Mina protocol and has a lot of Mina-specific logic This library SHOULD not be treated as a general-purpose BluePallas ciphersuite, but rather as a Mina-specific implementation.
Tuple Fields
0: PhantomData<M>Trait Implementations
Sourceimpl<M> Ciphersuite for BluePallas<M>where
M: ChallengeMessage,
impl<M> Ciphersuite for BluePallas<M>where
M: ChallengeMessage,
Sourcefn pre_commitment_aggregate<'a>(
signing_package: &'a SigningPackage<Self>,
binding_factor_list: &'a BindingFactorList<Self>,
) -> Result<Cow<'a, SigningPackage<Self>>, Error<Self>>
fn pre_commitment_aggregate<'a>( signing_package: &'a SigningPackage<Self>, binding_factor_list: &'a BindingFactorList<Self>, ) -> Result<Cow<'a, SigningPackage<Self>>, Error<Self>>
This performs the same functionality as Self::pre_commitment_sign, but instead only
negates commitments because the coordinator is not able to receive any nonces.
Naturally, this is called by the coordinator in the crate::aggregate function.
Sourcefn pre_commitment_sign<'a>(
signing_package: &'a SigningPackage<Self>,
signing_nonces: &'a SigningNonces<Self>,
binding_factor_list: &'a BindingFactorList<Self>,
) -> Result<(Cow<'a, SigningPackage<Self>>, Cow<'a, SigningNonces<Self>>), Error<Self>>
fn pre_commitment_sign<'a>( signing_package: &'a SigningPackage<Self>, signing_nonces: &'a SigningNonces<Self>, binding_factor_list: &'a BindingFactorList<Self>, ) -> Result<(Cow<'a, SigningPackage<Self>>, Cow<'a, SigningNonces<Self>>), Error<Self>>
This functions computes the group commitment and checks whether the y-coordinate of the
group commitment is even, as required by the Mina protocol.
If the group commitment is not even, it negates the nonces and commitments
This will be called by each individual signer during round2::sign
Sourceconst ID: &'static str = CONTEXT_STRING
const ID: &'static str = CONTEXT_STRING
Sourcetype Group = PallasGroup
type Group = PallasGroup
Sourcetype HashOutput = [u8; 32]
type HashOutput = [u8; 32]
Sourcetype SignatureSerialization = [u8; 32]
type SignatureSerialization = [u8; 32]
Group::ElementSerialization +
Group::ScalarSerializationSourcefn HDKG(m: &[u8]) -> Option<<<Self::Group as Group>::Field as Field>::Scalar>
fn HDKG(m: &[u8]) -> Option<<<Self::Group as Group>::Field as Field>::Scalar>
Sourcefn HID(m: &[u8]) -> Option<<<Self::Group as Group>::Field as Field>::Scalar>
fn HID(m: &[u8]) -> Option<<<Self::Group as Group>::Field as Field>::Scalar>
Sourcefn challenge(
r: &Element<Self>,
verifying_key: &VerifyingKey<Self>,
message: &[u8],
) -> Result<Challenge<Self>, Error<Self>>
fn challenge( r: &Element<Self>, verifying_key: &VerifyingKey<Self>, message: &[u8], ) -> Result<Challenge<Self>, Error<Self>>
round2::sign()] and [crate::aggregate()].fn single_sign<R>(
signing_key: &SigningKey<Self>,
rng: R,
message: &[u8],
) -> Signature<Self>
fn single_sign<R>( signing_key: &SigningKey<Self>, rng: R, message: &[u8], ) -> Signature<Self>
SigningKey]. Called
by [SigningKey::sign()]. This is not used by FROST. Can be overridden
if required which is useful if FROST signing has been changed by the
other Ciphersuite trait methods and regular signing should be changed
accordingly to match.fn verify_signature(
message: &[u8],
signature: &Signature<Self>,
public_key: &VerifyingKey<Self>,
) -> Result<(), Error<Self>>
fn verify_signature( message: &[u8], signature: &Signature<Self>, public_key: &VerifyingKey<Self>, ) -> Result<(), Error<Self>>
VerifyingKey::verify()]. The default implementation uses the
“cofactored” equation (it multiplies by the cofactor returned by
Group::cofactor()). Read morefn pre_sign<'a>(
signing_package: &'a SigningPackage<Self>,
signer_nonces: &'a SigningNonces<Self>,
key_package: &'a KeyPackage<Self>,
) -> Result<(Cow<'a, SigningPackage<Self>>, Cow<'a, SigningNonces<Self>>, Cow<'a, KeyPackage<Self>>), Error<Self>>
fn pre_sign<'a>( signing_package: &'a SigningPackage<Self>, signer_nonces: &'a SigningNonces<Self>, key_package: &'a KeyPackage<Self>, ) -> Result<(Cow<'a, SigningPackage<Self>>, Cow<'a, SigningNonces<Self>>, Cow<'a, KeyPackage<Self>>), Error<Self>>
round2::sign()] inputs. The default
implementation returns them as-is. Cow is used so implementations
can choose to return the same passed reference or a modified clone.fn pre_aggregate<'a>(
signing_package: &'a SigningPackage<Self>,
signature_shares: &'a BTreeMap<Identifier<Self>, SignatureShare<Self>>,
public_key_package: &'a PublicKeyPackage<Self>,
) -> Result<(Cow<'a, SigningPackage<Self>>, Cow<'a, BTreeMap<Identifier<Self>, SignatureShare<Self>>>, Cow<'a, PublicKeyPackage<Self>>), Error<Self>>
fn pre_aggregate<'a>( signing_package: &'a SigningPackage<Self>, signature_shares: &'a BTreeMap<Identifier<Self>, SignatureShare<Self>>, public_key_package: &'a PublicKeyPackage<Self>, ) -> Result<(Cow<'a, SigningPackage<Self>>, Cow<'a, BTreeMap<Identifier<Self>, SignatureShare<Self>>>, Cow<'a, PublicKeyPackage<Self>>), Error<Self>>
crate::aggregate()] and
[crate::verify_signature_share()] inputs. In the latter case, “dummy”
container BTreeMap and PublicKeyPackage are passed with the relevant
values (PublicKeyPackage.min_signers will be None). The default
implementation returns them as-is. Cow is used so implementations
can choose to return the same passed reference or a modified clone.fn pre_verify<'a>(
msg: &'a [u8],
signature: &'a Signature<Self>,
public_key: &'a VerifyingKey<Self>,
) -> Result<(Cow<'a, [u8]>, Cow<'a, Signature<Self>>, Cow<'a, VerifyingKey<Self>>), Error<Self>>
fn pre_verify<'a>( msg: &'a [u8], signature: &'a Signature<Self>, public_key: &'a VerifyingKey<Self>, ) -> Result<(Cow<'a, [u8]>, Cow<'a, Signature<Self>>, Cow<'a, VerifyingKey<Self>>), Error<Self>>
VerifyingKey::verify()] inputs. The default
implementation returns them as-is. Cow is used so implementations
can choose to return the same passed reference or a modified clone.fn generate_nonce<R>(
rng: &mut R,
) -> (<<Self::Group as Group>::Field as Field>::Scalar, <Self::Group as Group>::Element)
fn generate_nonce<R>( rng: &mut R, ) -> (<<Self::Group as Group>::Field as Field>::Scalar, <Self::Group as Group>::Element)
SigningKey] for regular (non-FROST) signing and internally by the DKG
to generate proof-of-knowledge signatures.round2::sign()].crate::aggregate()] if
cheater detection is enabled.fn serialize_signature(
signature: &Signature<Self>,
) -> Result<Vec<u8>, Error<Self>>
fn serialize_signature( signature: &Signature<Self>, ) -> Result<Vec<u8>, Error<Self>>
Ciphersuite::SignatureSerialization in bytes. Read morefn deserialize_signature(bytes: &[u8]) -> Result<Signature<Self>, Error<Self>>
fn deserialize_signature(bytes: &[u8]) -> Result<Signature<Self>, Error<Self>>
fn post_dkg(
key_package: KeyPackage<Self>,
public_key_package: PublicKeyPackage<Self>,
) -> Result<(KeyPackage<Self>, PublicKeyPackage<Self>), Error<Self>>
fn post_dkg( key_package: KeyPackage<Self>, public_key_package: PublicKeyPackage<Self>, ) -> Result<(KeyPackage<Self>, PublicKeyPackage<Self>), Error<Self>>
fn post_generate(
secret_shares: BTreeMap<Identifier<Self>, SecretShare<Self>>,
public_key_package: PublicKeyPackage<Self>,
) -> Result<(BTreeMap<Identifier<Self>, SecretShare<Self>>, PublicKeyPackage<Self>), Error<Self>>
fn post_generate( secret_shares: BTreeMap<Identifier<Self>, SecretShare<Self>>, public_key_package: PublicKeyPackage<Self>, ) -> Result<(BTreeMap<Identifier<Self>, SecretShare<Self>>, PublicKeyPackage<Self>), Error<Self>>
Sourceimpl<M> Clone for BluePallas<M>
impl<M> Clone for BluePallas<M>
Sourceimpl<M: Debug> Debug for BluePallas<M>
impl<M: Debug> Debug for BluePallas<M>
Sourceimpl<M: PartialEq> PartialEq for BluePallas<M>
impl<M: PartialEq> PartialEq for BluePallas<M>
impl<M> Copy for BluePallas<M>
impl<M: Eq> Eq for BluePallas<M>
impl<M> StructuralPartialEq for BluePallas<M>